==> vite-plugin-proxy-0.4.1.tgz: package/src/middlewares/detect-dev-server.js <==
import logger from '../logger.js'
import http2Client from 'http2-client'

const getProtocol = config => config.https ? 'https:' : 'http:'

const getHostFromServer = (server) => {
  const { address, family } = server.address()
  if (family === 'IPv6' && address === '::') {
    logger.info('Replace "::" with "0.0.0.0"')
    return '0.0.0.0'
  }
  if (family === 'IPv6') logger.warn(`Detected IPv6 address ${address}. This might not work.`)
  return address
}

const findDevServer = async ({ protocol, hostnames, port, httpServer }) => {
  for (const host of hostnames) {
    try {
      const url = new URL(`${protocol}//${host}:${port}`)
      logger.trace(`Constructed url: "${url.href}". Let's see if its valid.`)
      await ping(url, { timeout: 50 })
      logger.info(`Detected dev-server on ${url.href}`)
      // TODO we might want to get additional configuration
      return { target: url, secure: false }
    } catch (err) {
      if (httpServer.isClosed) return
      logger.warn('Could not verify url')
    }
  }
}

const getPortFromServer = server => server.address()?.port || 80

const ping = (url, { timeout } = {}) => Promise.race([
  new Promise((resolve, reject) => {
    const { protocol, hostname, port } = url
    logger.info(`Send ping request to ${url}`)
    http2Client.request({
      protocol,
      hostname,
      port,
      path: '/ping',
      headers: {
        accept: '*/*',
        'ping-by-ox-proxy': true
      }
    }, res => {
      if (res.statusCode !== 200) return reject(new Error('Not found'))
      resolve()
    }).end()
  }),
  new Promise((resolve, reject) => setTimeout(() => reject(new Error('Request timed out')), timeout))
])

let devServerUrl

export const getDevServerURL = () => devServerUrl

export default function ({ frontends, httpServer, serverConfig }) {
  httpServer.once('close', () => {
    httpServer.isClosed = true
  })
  httpServer.once('listening', async () => {
    const protocol = getProtocol(serverConfig)
    const hostname = getHostFromServer(httpServer)
    const port = getPortFromServer(httpServer)
    const hostnames = [hostname, '127.0.0.1', 'localhost', '0.0.0.0']
      .filter((host, i, arr) => arr.indexOf(host) === i)

    // retry 5 times
    for (let i = 0; i < 5; i++) {
      const frontend = await findDevServer({ protocol, port, hostnames, httpServer })
      if (frontend) {
        devServerUrl = frontend.target
        frontends.unshift(frontend)
        return
      }
      if (httpServer.isClosed) return
      logger.trace(`Dev-server could not be detected. Try all hostnames "${hostnames.join(', ')}" again.`)
    }

    logger.fatal('Could not find a working local vite dev server.')
    if (!httpServer.isClosed) process.exit(1)
  })

  return async function (req, res, next) {
    // respond to ping requests
    if (req.headers['ping-by-ox-proxy']) {
      logger.info('Respond ping request')
      res.statusCode = 200
      res.end()
      return
    }

    next()
  }
}

==> vite-plugin-proxy-0.5.0-pre1.tgz: package/src/middlewares/detect-dev-server.js <==
import logger from '../logger.js'
import http2Client from 'http2-client'

const getProtocol = config => config.https ? 'https:' : 'http:'

const getHostFromServer = (server) => {
  const { address, family } = server.address()
  if (family === 'IPv6' && address === '::') {
    logger.info('Replace "::" with "0.0.0.0"')
    return '0.0.0.0'
  }
  if (family === 'IPv6') logger.warn(`Detected IPv6 address ${address}. This might not work.`)
  return address
}

const findDevServer = async ({ protocol, hostnames, port, httpServer }) => {
  for (const host of hostnames) {
    try {
      const url = new URL(`${protocol}//${host}:${port}`)
      logger.trace(`Constructed url: "${url.href}". Let's see if its valid.`)
      await ping(url, { timeout: 50 })
      logger.info(`Detected dev-server on ${url.href}`)
      // TODO we might want to get additional configuration
      return { target: url, secure: false }
    } catch (err) {
      if (httpServer.isClosed) return
      logger.warn(`Could not verify url: ${protocol}//${host}:${port}`)
    }
  }
}

const getPortFromServer = server => server.address()?.port || 80

const ping = (url, { timeout } = {}) => Promise.race([
  new Promise((resolve, reject) => {
    const { protocol, hostname, port } = url
    logger.info(`Send ping request to ${url}`)
    http2Client.request({
      protocol,
      hostname,
      port,
      path: '/ping',
      headers: {
        accept: '*/*',
        'ping-by-ox-proxy': true
      }
    }, res => {
      if (res.statusCode !== 200) return reject(new Error('Not found'))
      resolve()
    }).end()
  }),
  new Promise((resolve, reject) => setTimeout(() => reject(new Error('Request timed out')), timeout))
])

let devServerUrl

export const getDevServerURL = () => devServerUrl

export default function ({ frontends, httpServer, serverConfig }) {
  httpServer.once('close', () => {
    httpServer.isClosed = true
  })
  const setupPromise = new Promise(resolve => {
    httpServer.once('listening', async () => {
      const protocol = getProtocol(serverConfig)
      const hostname = getHostFromServer(httpServer)
      const port = getPortFromServer(httpServer)
      const hostnames = [hostname, '127.0.0.1', 'localhost', '0.0.0.0']
        .filter((host, i, arr) => arr.indexOf(host) === i)

      // retry 5 times
      for (let i = 0; i < 5; i++) {
        const frontend = await findDevServer({ protocol, port, hostnames, httpServer })
        if (frontend) {
          devServerUrl = frontend.target
          frontends.unshift(frontend)
          logger.trace(`Added ${frontend.target} to frontends.`)
          return resolve()
        }
        if (httpServer.isClosed) return resolve()
        logger.trace(`Dev-server could not be detected. Try all hostnames "${hostnames.join(', ')}" again.`)
      }

      logger.fatal('Could not find a working local vite dev server.')
      // if (!httpServer.isClosed) process.exit(1)
    })
  })

  return async function (req, res, next) {
    // respond to ping requests
    if (req.headers['ping-by-ox-proxy']) {
      logger.info('Respond ping request')
      res.statusCode = 200
      res.end()
      return
    }

    await setupPromise
    next()
  }
}
